      program test_interpolate
      implicit none

! tests the abundance interpolation

! declare
      integer            :: i
      integer, parameter :: niso = 7
      double precision   :: twant, dwant, ywant, adiab(niso), plaw(niso)


! get the input
100   write(6,*) 'give temperature density ye, zeros to stop =>'
      read(5,*) twant,dwant,ywant
      if (twant .eq. 0.0) stop 'normal termination'

! bound the input
      if (twant .lt. 4.0e9 .or. twant .gt. 1.0e10) then
       write(6,*) 'temperature must be between 4.0e9 and 1.0e10 k; try again'
       goto 100
      end if
      if (dwant .lt. 1.0e4 .or. dwant .gt. 1.0e10) then
       write(6,*) 'density must be between 1.0e4 and 1.0e10 g/cc; try again'
       goto 100
      end if
      if (ywant .lt. 0.0484 .or. ywant .gt. 0.506) then
       write(6,*) 'baryon fraction Ye must be between 0.484 and 0.506; try again'
       goto 100
      end if


! interpolate
      call interpolate_abundances(twant,dwant,ywant, &
                                  niso,adiab,plaw)

! report the output
      write(6,09) 'temp','den','ti44','ni56','he4','fe54','c12','o16','si28'
09    format(1x,t5,a,t17,a,t29,a,t41,a,t53,a,t65,a,t77,a,t89,a,t101,a,t113,a,t125,a,t137,a)
      write(6,10) twant,dwant,ywant,(adiab(i), i=1,niso),' exponential'
      write(6,10) twant,dwant,ywant,(plaw(i), i=1,niso),' power-law'
10    format(1x,1p10e12.3,a)

      goto 100
      end program test_interpolate





      subroutine interpolate_abundances(twant,dwant,ywant, &
                                        nn,adiab_want,plaw_want)
      implicit none
      save


! interpolate the abundances from the 
! parameterized thermodynamic profiles

! declare the pass
      integer          ::  nn
      double precision twant, dwant, ywant, adiab_want(nn), plaw_want(nn)


! local variables

! for the tables sizes
! 121 temperature points
! 121 density points
! 12 ye points
!  7 isotopes

      integer, parameter :: imax = 121, jmax = 121, kmax=12, niso=7
      double precision   :: den(imax),temp(jmax),ye(kmax)


! for storing the ti44 abundance tables
      integer          :: i,j,k
      double precision :: adiab(imax,jmax,kmax,niso), & 
                          plaw(imax,jmax,kmax,niso)

! for the interpolations
      integer           :: iat,jat,kat
      integer,parameter :: morder = 2
      double precision  :: tlo,thi,tstp,tstpi,dlo,dhi,dstp,dstpi, &
                           ylo,yhi,ystp,ystpi,t,d

! initialize ifirst
      integer           :: ifirst = 0 


! do the table initialization and read only one

      if (ifirst .eq. 0) then
       ifirst = 1

! table limits, temperature and desnity in log space
       tlo   = 4.0d0 
       thi   = 10.0d0
       tstp  = (thi - tlo)/dfloat(jmax-1)
       tstpi = 1.0d0/tstp
       dlo   = 4.0d0
       dhi   = 10.0d0
       dstp  = (dhi - dlo)/dfloat(imax-1)
       dstpi = 1.0d0/dstp
       ylo   = 0.484d0
       yhi   = 0.506d0
       ystp  = (yhi - ylo)/dfloat(kmax-1)
       ystpi = 1.0d0/ystp


! read the tables
! columns are temp den ti44 ni56 he4 fe54 c12 o16 si28 time
       do k=1,kmax
        ye(k) = ylo + (k-1)*ystp
        if (k .eq. 1) then
         open(unit=19,file='AD1_ye0484.dat',status='old')
         open(unit=20,file='PL2_ye0484.dat',status='old')
        else if (k .eq. 2) then
         open(unit=19,file='AD1_ye0486.dat',status='old')
         open(unit=20,file='PL2_ye0486.dat',status='old')
        else if (k .eq. 3) then
         open(unit=19,file='AD1_ye0488.dat',status='old')
         open(unit=20,file='PL2_ye0488.dat',status='old')
        else if (k .eq. 4) then
         open(unit=19,file='AD1_ye0490.dat',status='old')
         open(unit=20,file='PL2_ye0490.dat',status='old')
        else if (k .eq. 5) then
         open(unit=19,file='AD1_ye0492.dat',status='old')
         open(unit=20,file='PL2_ye0492.dat',status='old')
        else if (k .eq. 6) then
         open(unit=19,file='AD1_ye0494.dat',status='old')
         open(unit=20,file='PL2_ye0494.dat',status='old')
        else if (k .eq. 7) then
         open(unit=19,file='AD1_ye0496.dat',status='old')
         open(unit=20,file='PL2_ye0496.dat',status='old')
        else if (k .eq. 8) then
         open(unit=19,file='AD1_ye0498.dat',status='old')
         open(unit=20,file='PL2_ye0498.dat',status='old')
        else if (k .eq. 9) then
         open(unit=19,file='AD1_ye0500.dat',status='old')
         open(unit=20,file='PL2_ye0500.dat',status='old')
        else if (k .eq. 10) then
         open(unit=19,file='AD1_ye0502.dat',status='old')
         open(unit=20,file='PL2_ye0502.dat',status='old')
        else if (k .eq. 11) then
         open(unit=19,file='AD1_ye0504.dat',status='old')
         open(unit=20,file='PL2_ye0504.dat',status='old')
        else if (k .eq. 12) then
         open(unit=19,file='AD1_ye0506.dat',status='old')
         open(unit=20,file='PL2_ye0506.dat',status='old')
        end if

        do j=1,jmax
         den(j) = 10.0d0**(dlo + (j-1)*dstp)
         do i=1,imax
          temp(i) = (tlo + (i-1)*tstp) * 1.0d9
          read(19,*) t, d, adiab(i,j,k,1),adiab(i,j,k,2), & 
                     adiab(i,j,k,3),adiab(i,j,k,4),adiab(i,j,k,5), & 
                     adiab(i,j,k,6),adiab(i,j,k,7)
          read(20,*) t, d, plaw(i,j,k,1), plaw(i,j,k,2), & 
                     plaw(i,j,k,3), plaw(i,j,k,4), plaw(i,j,k,5), & 
                     plaw(i,j,k,6), plaw(i,j,k,7)
         enddo
        enddo
        close(unit=19)
        close(unit=20)
       enddo

! end of initialization
      end if


! hash locate this temperature, density, and ye
      iat = int((twant*1.0d-9 - tlo)*tstpi) + 1
      iat = max(1,min(iat,imax-1))

      jat = int((log10(dwant) - dlo)*dstpi) + 1
      jat = max(1,min(jat,jmax-1))

      kat = int((ywant - ylo)*ystpi) + 1
      kat = max(1,min(kat,kmax-1))


! for the adiabatic exponential profiles
      do i=1,nn

        call hardwire_linear3(temp(iat),den(jat),ye(kat),adiab(iat,jat,kat,i), & 
                              morder,morder,morder, &
                              imax,jmax,kmax, &
                              twant,dwant,ywant,adiab_want(i))
      end do

! for the power-law profiles
      do i=1,nn

       call hardwire_linear3(temp(iat),den(jat),ye(kat),plaw(iat,jat,kat,i), & 
                             morder,morder,morder, &
                             imax,jmax,kmax, &
                             twant,dwant,ywant,plaw_want(i))
      end do

      return
      end subroutine interpolate_abundances



      subroutine hardwire_linear3(x1a,x2a,x3a,ya,i1a,i2a,i3a, &
                                  np1,np2,np3, &
                                  x1,x2,x3,answer)
      implicit none

! given arrays x1a(1:i1a), x2a(1:i2a), and x3a(1:i3a) of
! independent variables; an array of function values ya(i1a,i2a,i3a)
! which has physical dimensions ya(np1,np2,np3),
! tabulated at the cartesian grid points defined by x1a, x2a and x3a;
! values of the desired point x1, x2, and x3, then this routine returns an
! interpolated function value answer.

! declare the pass
      integer          :: i1a,i2a,i3a,np1,np2,np3
      double precision :: x1a(i1a),x2a(i2a),x3a(i3a), &
                          ya(np1,np2,np3),x1,x2,x3,answer

! local variables
! imax is the maximum order
      integer            :: i,j,k
      integer, parameter :: imax = 10
      double precision   :: yntmp(imax), ymtmp(imax), yotmp(imax)

! check
      if (i1a .gt. imax .or. i2a .gt. imax  .or. i3a .gt. imax) stop 'imax too small in hardwire_linear3'


! loop over x3a and x2a
      do k=1,i3a
       do j=1,i2a

! store the x1a row, interpolate in x1a and store for x2a
        do i=1,i1a
         yotmp(i) = ya(i,j,k)
        enddo
        call hardwire_linear(x1a,yotmp,i1a,x1,yntmp(j))
       enddo

! interpolate in x2a and store for x3a
       call hardwire_linear(x2a,yntmp,i2a,x2,ymtmp(k))
      enddo

! interpolate in x3a
      call hardwire_linear(x3a,ymtmp,i3a,x3,answer)
      return
      end subroutine hardwire_linear3



      subroutine hardwire_linear(xtab,ytab,n,xin,yout)
      implicit none

! implements a hardwired linear interpolation

! declare the pass
      integer          :: n
      double precision :: xtab(n),ytab(n),xin,yout

! local variables
      double precision x1,x2,denom,alfa,beta

      x1    = xtab(1)
      x2    = xtab(2)
      denom = 1.0d0/(x1-x2)
      alfa  = (xin - x2)*denom
      beta  = (x1 - xin)*denom
      yout  = alfa*ytab(1) + beta*ytab(2)

      return
      end subroutine hardwire_linear
